<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use stdClass;

/**
 * Class Redirects
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Redirects extends AbstractExporter
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo
    )
    {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
    }


    public function getCollections(): iterable
    {
        yield 'redirects';
    }

    public function execute()
    {
        $bulkData = [];
        $query = "SELECT * FROM `filoblu_refilo_redirect`";

        $result = $this->resourceConnection->getConnection()->fetchAll($query);

        foreach ($result as $item) {
            $redirect = new stdClass();

            $redirect->_id = (int)$item['id'];
            $redirect->host = $item['host'];
            $redirect->request_url = $item['request_url'];
            $redirect->target_url = $item['target_url'];
            $redirect->redirect_code = (int)$item['redirect_code'];
            $bulkData[] = $redirect;
        }

        if (!empty($bulkData)) {
            try {
               $this->getConnection()->update(
                    '_id',
                    $bulkData,
                    'redirects',
                    true,
                    $this->getDatabase()
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        } else {
            try {
                $this->getConnection()->flushCollection(
                    'redirects',
                   $this->getDatabase()
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }

    }
}
