<?php

namespace FiloBlu\Refilo\Plugins\Api;

use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Framework\Event\ManagerInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class AccountManagementInterface
 * @package FiloBlu\Refilo\Plugins\Api
 */
class AccountManagementInterface
{
    /**
     * @var ManagerInterface
     */
    private $eventManager;
    /**
     * @var \FiloBlu\Esb\Api\ObjectTypeServiceInterface
     */
    private $objectTypeService;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var \Magento\Framework\Webapi\Rest\Request
     */
    private $request;

    /**
     * AccountManagementInterface constructor.
     * @param ManagerInterface $eventManager
     * @param \FiloBlu\Esb\Api\ObjectTypeServiceInterface $objectTypeService
     * @param StoreManagerInterface $storeManager
     * @param \Magento\Framework\Webapi\Rest\Request $request
     */
    public function __construct(
        ManagerInterface $eventManager,
        \FiloBlu\Esb\Api\ObjectTypeServiceInterface $objectTypeService,
        StoreManagerInterface $storeManager,
        \Magento\Framework\Webapi\Rest\Request $request
    )
    {
        $this->eventManager = $eventManager;
        $this->objectTypeService = $objectTypeService;
        $this->storeManager = $storeManager;
        $this->request = $request;
    }

    /**
     * @param $subject
     * @param $result
     * @param CustomerInterface $customer
     * @return mixed
     * @throws \Exception
     */
    public function afterCreateAccount($subject, $result, CustomerInterface $customer)
    {
        $this->eventManager->dispatch(
            'customer_register_success',
            [
                'account_controller' => null,
                'customer'           => $customer
            ]
        );

        // TODO : Merge fields coming from $request
        if ((bool)$result->getExtensionAttributes()->getIsSubscribed() === true) {
            $store = $this->storeManager->getStore($result->getStoreId());
            $this->objectTypeService->push('FiloBlu_Esb::form_event',
                [
                    'form_type' => 'newsletter',
                    'fields'    => [
                        'email'              => $result->getEmail(),
                        'firstname'          => $result->getFirstname(),
                        'lastname'           => $result->getLastname(),
                        'dob'                => $result->getDob(),
                        'gender'             => $result->getGender(),
                        'collections'        => '',
                        'phone'              => '',
                        'country'            => '',
                        'source'             => 'ACCOUNT',
                        'newsletter_privacy' => 'true',
                        'files'              => [],
                        'store'              => $store->getCode()
                    ]
                ]);
        }

        return $result;
    }
}
